@@ -51,7 +51,6 @@ package com.openexchange.authentication.imap.impl;
 
 import static com.openexchange.authentication.LoginExceptionCodes.INVALID_CREDENTIALS;
 import static com.openexchange.authentication.LoginExceptionCodes.UNKNOWN;
-import static com.openexchange.authentication.imap.osgi.ImapAuthServiceRegistry.getServiceRegistry;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
@@ -64,22 +63,23 @@ import javax.mail.Session;
 import javax.mail.Store;
 import javax.mail.internet.idn.IDNA;
 import javax.security.auth.login.LoginException;
-import org.apache.commons.logging.Log;
 import com.openexchange.authentication.Authenticated;
 import com.openexchange.authentication.AuthenticationService;
 import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.authentication.LoginInfo;
+import com.openexchange.config.ConfigurationService;
 import com.openexchange.configuration.ConfigurationException;
 import com.openexchange.context.ContextService;
 import com.openexchange.exception.OXException;
 import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextStorage;
 import com.openexchange.java.Streams;
-import com.openexchange.log.LogFactory;
+import com.openexchange.java.Strings;
 import com.openexchange.mail.api.MailConfig.LoginSource;
 import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.mailaccount.MailAccountStorageService;
+import com.openexchange.server.ServiceLookup;
 import com.openexchange.user.UserService;
 
 public class IMAPAuthentication implements AuthenticationService {
@@ -120,7 +120,7 @@ public class IMAPAuthentication implements AuthenticationService {
         }
     }
 
-    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(IMAPAuthentication.class));
+    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(IMAPAuthentication.class);
 
     private static Properties props;
 
@@ -131,11 +131,14 @@ public class IMAPAuthentication implements AuthenticationService {
      */
     private static final String CHARENC_ISO8859 = "ISO-8859-1";
 
+    private final ServiceLookup services;
+
     /**
      * Default constructor.
      */
-    public IMAPAuthentication() {
+    public IMAPAuthentication(final ServiceLookup services) {
         super();
+        this.services = services;
     }
 
     // /**
@@ -180,7 +183,7 @@ public class IMAPAuthentication implements AuthenticationService {
                 try {
                     password = new String(password.getBytes(authenc), CHARENC_ISO8859);
                 } catch (final UnsupportedEncodingException e) {
-                    LOG.error(e.getMessage(), e);
+                    LOG.error("", e);
                     throw LoginExceptionCodes.COMMUNICATION.create(e);
                 }
             }
@@ -209,9 +212,9 @@ public class IMAPAuthentication implements AuthenticationService {
                 port = Integer.parseInt((String) props.get(PropertyNames.IMAP_PORT.name));
             }
 
-            LOG.debug("Using imap server: " + host);
-            LOG.debug("Using imap port: " + port);
-            LOG.debug("Using full login info: " + use_full_login);
+            LOG.debug("Using imap server: {}", host);
+            LOG.debug("Using imap port: {}", port);
+            LOG.debug("Using full login info: {}", use_full_login);
 
             // set imap username
             if (use_full_login) {
@@ -224,7 +227,7 @@ public class IMAPAuthentication implements AuthenticationService {
             // Added by cutmasta
             boolean USE_IMAPS = false;
             if ("true".equalsIgnoreCase(props.getProperty(PropertyNames.USE_MULTIPLE.name))) {
-	            final ContextService contextService = getServiceRegistry().getService(ContextService.class, true);
+	            final ContextService contextService = services.getService(ContextService.class);
 
 	            final int ctxId = contextService.getContextId(splitted[0]);
 	            if (ContextStorage.NOT_FOUND == ctxId) {
@@ -232,7 +235,7 @@ public class IMAPAuthentication implements AuthenticationService {
 	            }
 	            final Context ctx = contextService.getContext(ctxId);
 
-	            final UserService userService = getServiceRegistry().getService(UserService.class, true);
+	            final UserService userService = services.getService(UserService.class);
 	            final int userId;
 	            try {
 	                userId = userService.getUserId(uid, ctx);
@@ -244,8 +247,7 @@ public class IMAPAuthentication implements AuthenticationService {
 	            /*
 	             * Load primary account and check its protocol to be IMAP
 	             */
-	            final MailAccount defaultMailAccount =
-	                getServiceRegistry().getService(MailAccountStorageService.class, true).getDefaultMailAccount(userId, ctxId);
+	            final MailAccount defaultMailAccount = services.getService(MailAccountStorageService.class).getDefaultMailAccount(userId, ctxId);
 	            final String mailProtocol = defaultMailAccount.getMailProtocol();
 	            if (!mailProtocol.toLowerCase().startsWith("imap")) {
 	                throw UNKNOWN.create(new StringBuilder(128).append(
@@ -270,7 +272,7 @@ public class IMAPAuthentication implements AuthenticationService {
 	            host = IDNA.toASCII(defaultMailAccount.getMailServer());
 	            port = defaultMailAccount.getMailPort();
 	            USE_IMAPS = defaultMailAccount.isMailSecure();
-	            LOG.debug("Parsed IMAP Infos: " + (USE_IMAPS ? "imaps" : "imap") + " " + host + " " + port + "  (" + userId + "@" + ctxId + ")");
+	            LOG.debug("Parsed IMAP Infos: {} {} {}  ({}@{})", (USE_IMAPS ? "imaps" : "imap"), host, port, userId, ctxId);
 
 
             } else {
@@ -298,6 +300,24 @@ public class IMAPAuthentication implements AuthenticationService {
                  * Needed for JavaMail >= 1.4
                  */
                 // Security.setProperty("ssl.SocketFactory.provider", socketFactoryClass);
+                /*
+                 * Specify SSL protocols
+                 */
+                {
+                    final ConfigurationService configuration = services.getService(ConfigurationService.class);
+                    final String sslProtocols = configuration.getProperty("com.openexchange.imap.ssl.protocols", "SSLv3 TLSv1").trim();
+                    imapprops.put("mail.imap.ssl.protocols", sslProtocols);
+                }
+                /*
+                 * Specify SSL cipher suites
+                 */
+                {
+                    final ConfigurationService configuration = services.getService(ConfigurationService.class);
+                    final String cipherSuites = configuration.getProperty("com.openexchange.imap.ssl.ciphersuites", "").trim();
+                    if (false == Strings.isEmpty(cipherSuites)) {
+                        imapprops.put("mail.imap.ssl.ciphersuites", cipherSuites);
+                    }
+                }
             } else {
                 /*
                  * Enables the use of the STARTTLS command (if supported by the server) to switch the connection to a TLS-protected connection.
@@ -314,7 +334,21 @@ public class IMAPAuthentication implements AuthenticationService {
                 /*
                  * Specify SSL protocols
                  */
-                imapprops.put("mail.imap.ssl.protocols", "SSLv3 TLSv1");
+                {
+                    final ConfigurationService configuration = services.getService(ConfigurationService.class);
+                    final String sslProtocols = configuration.getProperty("com.openexchange.imap.ssl.protocols", "SSLv3 TLSv1").trim();
+                    imapprops.put("mail.imap.ssl.protocols", sslProtocols);
+                }
+                /*
+                 * Specify SSL cipher suites
+                 */
+                {
+                    final ConfigurationService configuration = services.getService(ConfigurationService.class);
+                    final String cipherSuites = configuration.getProperty("com.openexchange.imap.ssl.ciphersuites", "").trim();
+                    if (false == Strings.isEmpty(cipherSuites)) {
+                        imapprops.put("mail.imap.ssl.ciphersuites", cipherSuites);
+                    }
+                }
                 // imapProps.put("mail.imap.ssl.enable", "true");
                 /*
                  * Needed for JavaMail >= 1.4
@@ -328,14 +362,14 @@ public class IMAPAuthentication implements AuthenticationService {
             imapconnection = session.getStore("imap");
             // try to connect with the credentials set above
             imapconnection.connect(host, port, user, password);
-            LOG.info("Imap authentication for user " + user + " successful on host " + host + ":" + port);
+            LOG.info("Imap authentication for user {} successful on host {}:{}", user, host, port);
 
             /*
              * Set the context of the user, If full login was configured, we use the domain part as the context name/mapping entry. If NO
              * full login was configured, we assume that only 1 context is in the system which is named "defaultcontext".
              */
             if (use_full_login) {
-                LOG.debug("Using domain: " + splitted[0] + " as context name!");
+                LOG.debug("Using domain: {} as context name!", splitted[0]);
             } else {
                 LOG.debug("Using \"defaultcontext\" as context name!");
                 splitted[0] = "defaultcontext";
@@ -348,11 +382,11 @@ public class IMAPAuthentication implements AuthenticationService {
             LOG.error("Error setup initial imap envorinment!", e);
             throw LoginExceptionCodes.COMMUNICATION.create(e);
         } catch (final AuthenticationFailedException e) {
-            LOG.info("Authentication error on host " + host + ":" + port + " for user " + user, e);
+            LOG.info("Authentication error on host {}:{} for user {}", host, port, user, e);
             LOG.debug("Debug imap authentication", e);
             throw LoginExceptionCodes.INVALID_CREDENTIALS.create(e);
         } catch (final MessagingException e) {
-            LOG.info("Messaging error on host " + host + ":" + port + " for user " + user, e);
+            LOG.info("Messaging error on host {}:{} for user {}", host, port, user, e);
             LOG.debug("Debug imap error", e);
             throw LoginExceptionCodes.UNKNOWN.create(e, e.getMessage());
         } finally {
